#!/bin/bash

env | grep AHCP

die() {
    echo $1 >&2
    exit 1
}

findcmd() {
    type "$1" &> /dev/null || \
        die "Couldn't find $1, please install ${2:-it} or fix your path."
}

olsrd_pidfile=/var/run/ahcp_olsrd_pid
usage="Usage: $0 (start|stop)"
debuglevel=${AHCP_DEBUG_LEVEL:-1}

[ $# -eq 1 ] || die "$usage"

interface="$AHCP_INTERFACE"
if [ -z "$interface" ]; then die "No interface set"; fi

findcmd ip iproute
findcmd iwconfig "wireless-tools"
findcmd ipv6calc
findcmd olsrd

(ip -6 addr show dev lo | grep -q 'inet6') || \
    die "No IPv6 address on lo, please modprobe ipv6"

nodns=${AHCP_NAMESERVER:+1}
l="$(ip -0 addr show dev "$interface" | grep '^ *link/ether ' | head -n 1)"
mac="$(echo "$l" | sed 's|^ *link/ether \([0-9a-z:]*\) .*$|\1|')"
iid="$(ipv6calc -A geneui64 -I mac -O eui64 "$mac")"
prefix="$(echo $AHCP_PREFIX | \
          sed 's/^\([0-9a-z]*:[0-9a-z]*:[0-9a-z]*:[0-9a-z]*\):.*$/\1/')"
address="$prefix:$iid"

nameserver_start() {
    if [ -z $nodns ]; then
        mv /etc/resolv.conf /etc/resolv.conf.orig
        for n in $AHCP_NAMESERVER; do
            echo "nameserver $n" >> /etc/resolv.conf
        done
    fi
}

nameserver_stop() {
    if [ -z $nodns ]; then
        mv /etc/resolv.conf.orig /etc/resolv.conf
    fi
}    

start_none() {
    ip link set up dev "$interface" || die "Couldn't up interface"
    ip -6 addr add "$address" dev "$interface"
}

stop_none() {
    ip -6 addr del "$address" dev "$interface"
    ip link set down dev "$interface"
}

start_static() {
    ip link set up dev "$interface" || die "Couldn't up interface"
    ip -6 addr add "$address" dev "$interface"
    for gw in $AHCP_STATIC_DEFAULT_GATEWAY; do
        ip -6 route add ::/0 via $gw
    done
}

stop_static() {
    for gw in $AHCP_STATIC_DEFAULT_GATEWAY; do
        ip -6 route del ::/0 via $gw
    done
    ip -6 addr del "$address" dev "$interface"
    ip link set down dev "$interface"
}

start_olsr() {
    multicast="$AHCP_OLSR_MULTICAST_ADDRESS"

    conf_filename=/var/lib/ahcp-olsrd-"$AHCP_DAEMON_PID".conf


    cat > "$conf_filename" <<EOF
DebugLevel $debuglevel

IpVersion 6

AllowNoInt yes

Interface "$interface" {
    Ip6AddrType global
    Ip6MulticastSite $multicast
    Ip6MulticastGlobal $multicast
}
EOF

    if [ ! -z "$AHCP_HNA6" ]; then
        cat >> "$conf_filename" <<EOF
Hna6 {
    $AHCP_HNA6
}
EOF
    fi

    ip link set up dev "$interface" || die "Couldn't up interface"
    ip -6 addr add "$address" dev "$interface" || die "Couldn't add address"
    nameserver_start
    olsrd -f "$conf_filename" -nofork &
    echo $! > $olsrd_pidfile
}

stop_olsr() {
    conf_filename=/var/lib/ahcp-olsrd-"$AHCP_DAEMON_PID".conf

    kill $(cat "$olsrd_pidfile")

    ip -6 addr del "$address" dev "$interface"
    ip link set down dev "$interface"
    rm "$conf_filename"
    nameserver_stop
}

case $1 in
    start)
        case ${AHCP_ROUTING_PROTOCOL:-none} in
            none) start_none;;
            static) start_static;;
            OLSR) start_olsr;;
            *) die "Unknown routing protocol $AHCP_ROUTING_PROTOCOL";;
        esac;;
    stop)
        case ${AHCP_ROUTING_PROTOCOL:-none} in
            none) stop_none;;
            static) stop_static;;
            OLSR) stop_olsr;;
            *) die "Unknown routing protocol $AHCP_ROUTING_PROTOCOL";;
        esac;;
esac
